@@ -11,11 +11,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+# Save trace setting
+XTRACE=$(set +o | grep xtrace)
+set -o xtrace
+
 function container_runtime {
     if [[ ${CONTAINER_ENGINE} == 'crio' ]]; then
-        sudo podman "$@"
+        sudo podman "$@" || die $LINENO "Error when running podman command"
     else
-        docker "$@"
+        docker "$@" || die $LINENO "Error when running docker command"
     fi
 }
 
@@ -143,10 +147,8 @@ function generate_containerized_kuryr_resources {
     iniset "$KURYR_CONFIG" kubernetes controller_ha_port ${KURYR_CONTROLLER_HA_PORT}
 
     # NOTE(dulek): In the container the CA bundle will be mounted in a standard
-    # directory, so we need to modify that.
+    # directory
     iniset "$KURYR_CONFIG" neutron cafile /etc/ssl/certs/kuryr-ca-bundle.crt
-    iniset "$KURYR_CONFIG" kubernetes token_file /var/run/secrets/kubernetes.io/serviceaccount/token
-    iniset "$KURYR_CONFIG" kubernetes ssl_ca_crt_file /var/run/secrets/kubernetes.io/serviceaccount/ca.crt
 
     # Generate kuryr resources in k8s formats.
     local output_dir="${DATA_DIR}/kuryr-kubernetes"
@@ -188,7 +190,7 @@ function install_kuryr_cni {
 }
 
 function _cidr_range {
-  python - <<EOF "$1"
+  python3 - <<EOF "$1"
 import sys
 from netaddr import IPAddress, IPNetwork
 n = IPNetwork(sys.argv[1])
@@ -232,7 +234,7 @@ function create_k8s_api_service {
                              -c cidr -f value)
 
     fixed_ips=$(openstack port show kubelet-"${HOSTNAME}" -c fixed_ips -f value)
-    kubelet_iface_ip=$(python -c "print ${fixed_ips}[0]['ip_address']")
+    kubelet_iface_ip=$(python3 -c "print(${fixed_ips}[0]['ip_address'])")
 
     k8s_api_clusterip=$(_cidr_range "$service_cidr" | cut -f1)
 
@@ -272,8 +274,8 @@ function configure_neutron_defaults {
     local router
     local router_id
     local ext_svc_net_id
-    local ext_svc_subnet_id
-    local prot
+    local addrs_prefix
+    local subnetpool_name
 
     project_id=$(get_or_create_project \
         "$KURYR_NEUTRON_DEFAULT_PROJECT" default)
@@ -282,7 +284,35 @@ function configure_neutron_defaults {
 
     # If a subnetpool is not passed, we get the one created in devstack's
     # Neutron module
-    subnetpool_id=${KURYR_NEUTRON_DEFAULT_SUBNETPOOL_ID:-${SUBNETPOOL_V4_ID}}
+    KURYR_IPV6=$(trueorfalse False KURYR_IPV6)
+    if [ "$KURYR_IPV6" == "False" ]; then
+        export KURYR_ETHERTYPE=IPv4
+        subnetpool_id=${KURYR_NEUTRON_DEFAULT_SUBNETPOOL_ID:-${SUBNETPOOL_V4_ID}}
+    else
+        export KURYR_ETHERTYPE=IPv6
+        # NOTE(gryf): To not clash with subnets created by DevStack for IPv6,
+        # we create another subnetpool just for kuryr subnets.
+        # SUBNETPOOL_KURYR_V6_ID will be used in function configure_kuryr in
+        # case of namespace kuryr subnet driver.
+        # This is not required for IPv4, because DevStack is only adding a
+        # conflicting route for IPv6. On DevStack this route is opening public
+        # IPv6 network to be accessible from host, which doesn't have place in
+        # IPv4 net, because floating IPs are used instead.
+        IPV6_ID=$(uuidgen | sed s/-//g | cut -c 23- | \
+            sed -e "s/\(..\)\(....\)\(....\)/\1:\2:\3/")
+        addrs_prefix="fd${IPV6_ID}::/56"
+        subnetpool_name=${SUBNETPOOL_KURYR_NAME_V6}
+        SUBNETPOOL_KURYR_V6_ID=$(openstack \
+            --os-cloud devstack-admin \
+            --os-region "${REGION_NAME}" \
+            subnet pool create "${subnetpool_name}" \
+            --default-prefix-length "${SUBNETPOOL_SIZE_V6}" \
+            --pool-prefix "${addrs_prefix}" \
+            --share -f value -c id)
+        export SUBNETPOOL_KURYR_V6_ID
+        subnetpool_id=${KURYR_NEUTRON_DEFAULT_SUBNETPOOL_ID:-${SUBNETPOOL_KURYR_V6_ID}}
+    fi
+
     router=${KURYR_NEUTRON_DEFAULT_ROUTER:-$Q_ROUTER_NAME}
     if [ "$router" != "$Q_ROUTER_NAME" ]; then
         openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
@@ -332,14 +362,14 @@ function configure_neutron_defaults {
     openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
         security group rule create --project "$project_id" \
         --description "k8s service subnet allowed" \
-        --remote-ip "$service_cidr" --ethertype IPv4 --protocol tcp \
+        --remote-ip "$service_cidr" --ethertype "$KURYR_ETHERTYPE" --protocol tcp \
         "$service_pod_access_sg_id"
     # Since Octavia supports also UDP load balancing, we need to allow
     # also udp traffic
     openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
         security group rule create --project "$project_id" \
         --description "k8s service subnet UDP allowed" \
-        --remote-ip "$service_cidr" --ethertype IPv4 --protocol udp \
+        --remote-ip "$service_cidr" --ethertype "$KURYR_ETHERTYPE" --protocol udp \
         "$service_pod_access_sg_id"
 
     if [[ "$KURYR_K8S_OCTAVIA_MEMBER_MODE" == "L3" ]]; then
@@ -367,14 +397,14 @@ function configure_neutron_defaults {
         openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
             security group rule create --project "$project_id" \
             --description "k8s pod subnet allowed from k8s-pod-subnet" \
-            --remote-ip "$pod_cidr" --ethertype IPv4 --protocol tcp \
+            --remote-ip "$pod_cidr" --ethertype "$KURYR_ETHERTYPE" --protocol tcp \
             "$octavia_pod_access_sg_id"
         # Since Octavia supports also UDP load balancing, we need to allow
         # also udp traffic
         openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
             security group rule create --project "$project_id" \
             --description "k8s pod subnet allowed from k8s-pod-subnet" \
-            --remote-ip "$pod_cidr" --ethertype IPv4 --protocol udp \
+            --remote-ip "$pod_cidr" --ethertype "$KURYR_ETHERTYPE" --protocol udp \
             "$octavia_pod_access_sg_id"
         if [ -n "$sg_ids" ]; then
             sg_ids+=",${octavia_pod_access_sg_id}"
@@ -390,47 +420,7 @@ function configure_neutron_defaults {
         iniset "$KURYR_CONFIG" namespace_subnet pod_subnet_pool "$subnetpool_id"
         iniset "$KURYR_CONFIG" namespace_subnet pod_router "$router_id"
     fi
-    if [ "$KURYR_SG_DRIVER" == "namespace" ]; then
-        local allow_namespace_sg_id
-        local allow_default_sg_id
-        allow_namespace_sg_id=$(openstack --os-cloud devstack-admin \
-            --os-region "$REGION_NAME" \
-            security group create --project "$project_id" \
-            allow_from_namespace -f value -c id)
-        allow_default_sg_id=$(openstack --os-cloud devstack-admin \
-            --os-region "$REGION_NAME" \
-            security group create --project "$project_id" \
-            allow_from_default -f value -c id)
-
-        for prot in icmp tcp udp ;
-          do
-            openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
-              security group rule create --project "$project_id" \
-              --description "allow traffic from default namespace" \
-              --remote-group "$allow_namespace_sg_id" --ethertype IPv4 --protocol "$prot" \
-              "$allow_default_sg_id"
-
-            if [ "$prot" != "icmp" ] ; then
-              openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
-              security group rule create --project "$project_id" \
-              --description "allow traffic from namespaces at default namespace" \
-              --remote-group "$allow_default_sg_id" --ethertype IPv4 --protocol "$prot" \
-              "$allow_namespace_sg_id"
-            fi
-          done
-
-        # NOTE(ltomasbo): Some tempest test are using FIP and depends on icmp
-        # traffic being allowed to the pods. To enable these tests we permit
-        # icmp traffic from everywhere on the default namespace. Note tcp
-        # traffic will be dropped, just icmp is permitted.
-        openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
-            security group rule create --project "$project_id" \
-            --description "allow imcp traffic from everywhere to default namespace" \
-            --ethertype IPv4 --protocol icmp "$allow_namespace_sg_id"
-
-        iniset "$KURYR_CONFIG" namespace_sg sg_allow_from_namespaces "$allow_namespace_sg_id"
-        iniset "$KURYR_CONFIG" namespace_sg sg_allow_from_default "$allow_default_sg_id"
-    elif [[ "$KURYR_SG_DRIVER" == "policy" ]]; then
+    if [[ "$KURYR_SG_DRIVER" == "policy" ]]; then
         # NOTE(dulek): Using the default DevStack's SG is not enough to match
         # the NP specification. We need to open ingress to everywhere, so we
         # create allow-all group.
@@ -441,7 +431,7 @@ function configure_neutron_defaults {
         openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
           security group rule create --project "$project_id" \
           --description "allow all ingress traffic" \
-          --ethertype IPv4 --ingress --protocol any \
+          --ethertype "$KURYR_ETHERTYPE" --ingress --protocol any \
           "$allow_all_sg_id"
         if [ -n "$sg_ids" ]; then
             sg_ids+=",${allow_all_sg_id}"
@@ -451,7 +441,7 @@ function configure_neutron_defaults {
     fi
     iniset "$KURYR_CONFIG" neutron_defaults pod_security_groups "$sg_ids"
 
-    if [[ "$KURYR_SG_DRIVER" == "namespace" || "$KURYR_SG_DRIVER" == "policy" ]]; then
+    if [[ "$KURYR_SG_DRIVER" == "policy" ]]; then
         # NOTE(ltomasbo): As more security groups and rules are created, there
         # is a need to increase the quota for it
          openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
@@ -471,6 +461,7 @@ function configure_neutron_defaults {
     iniset "$KURYR_CONFIG" octavia_defaults member_mode "$KURYR_K8S_OCTAVIA_MEMBER_MODE"
     iniset "$KURYR_CONFIG" octavia_defaults sg_mode "$KURYR_K8S_OCTAVIA_SG_MODE"
     iniset "$KURYR_CONFIG" octavia_defaults enforce_sg_rules "$KURYR_ENFORCE_SG_RULES"
+    iniset "$KURYR_CONFIG" octavia_defaults lb_algorithm "$KURYR_LB_ALGORITHM"
     # Octavia takes a very long time to start the LB in the gate. We need
     # to tweak the timeout for the LB creation. Let's be generous and give
     # it up to 20 minutes.
@@ -627,7 +618,7 @@ function run_k8s_api {
         command+=(--restart=on-failure)
     fi
     command+=(${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}
-              /hyperkube apiserver
+              /hyperkube kube-apiserver
                 --service-cluster-ip-range=${cluster_ip_range}
                 --insecure-bind-address=0.0.0.0
                 --insecure-port=${KURYR_K8S_API_PORT}
@@ -657,14 +648,13 @@ function run_k8s_controller_manager {
         command+=(--restart=on-failure)
     fi
     command+=(${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}
-              /hyperkube controller-manager
+              /hyperkube kube-controller-manager
                 --master=$KURYR_K8S_API_URL
                 --service-account-private-key-file=/srv/kubernetes/server.key
                 --root-ca-file=/srv/kubernetes/ca.crt
                 --min-resync-period=3m
                 --v=$(get_k8s_log_level)
-                --logtostderr=true
-                --leader-elect=false)
+                --logtostderr=true)
 
     run_container kubernetes-controller-manager "${command[@]}"
 }
@@ -681,11 +671,10 @@ function run_k8s_scheduler {
         command+=(--restart=on-failure)
     fi
     command+=(${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}
-              /hyperkube scheduler
+              /hyperkube kube-scheduler
                 --master=$KURYR_K8S_API_URL
                 --v=$(get_k8s_log_level)
-                --logtostderr=true
-                --leader-elect=false)
+                --logtostderr=true)
 
     run_container kubernetes-scheduler "${command[@]}"
 }
@@ -759,7 +748,6 @@ function run_k8s_kubelet {
     sudo mkdir -p "${KURYR_HYPERKUBE_DATA_DIR}/"{kubelet,kubelet.cert}
     command="$KURYR_HYPERKUBE_BINARY kubelet\
         --kubeconfig=${HOME}/.kube/config \
-        --allow-privileged=true \
         --v=2 \
         --address=0.0.0.0 \
         --enable-server \
@@ -770,7 +758,7 @@ function run_k8s_kubelet {
         --root-dir=${KURYR_HYPERKUBE_DATA_DIR}/kubelet"
 
     if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
-        command+=" --cgroup-driver $(docker info -f '{{.CgroupDriver}}')"
+        command+=" --cgroup-driver $(docker info|awk '/Cgroup/ {print $NF}')"
     elif [[ ${CONTAINER_ENGINE} == 'crio' ]]; then
         local crio_conf
         crio_conf=/etc/crio/crio.conf
@@ -803,16 +791,7 @@ function run_k8s_kubelet {
     fi
 
     wait_for "Kubernetes API Server" "$KURYR_K8S_API_URL"
-    if [[ "$USE_SYSTEMD" = "True" ]]; then
-        # If systemd is being used, proceed as normal
-        run_process kubelet "$command" root root
-    else
-        # If screen is being used, there is a possibility that the devstack
-        # environment is on a stable branch. Older versions of run_process have
-        # a different signature. Sudo is used as a workaround that works in
-        # both older and newer versions of devstack.
-        run_process kubelet "sudo $command"
-    fi
+    run_process kubelet "$command" root root
 }
 
 function run_coredns {
@@ -850,7 +829,7 @@ EOF
     cat >> "${output_dir}/coredns.yml" << EOF
     }
 ---
-apiVersion: extensions/v1beta1
+apiVersion: apps/v1
 kind: Deployment
 metadata:
   name: coredns
@@ -875,7 +854,7 @@ spec:
       hostNetwork: true
       containers:
       - name: coredns
-        image: quay.io/kuryr/coredns:1.4.0
+        image: coredns/coredns:1.4.0
         imagePullPolicy: Always
         args: [ "-conf", "/etc/coredns/Corefile" ]
         volumeMounts:
@@ -896,7 +875,7 @@ EOF
 
 
 function run_kuryr_kubernetes {
-    local python_bin=$(which python)
+    local python_bin=$(which python3)
 
     if is_service_enabled openshift-master; then
         wait_for "OpenShift API Server" "${KURYR_K8S_API_ROOT}" \
@@ -919,60 +898,6 @@ function run_kuryr_daemon {
     run_process kuryr-daemon "$daemon_bin --config-file $KURYR_CONFIG" root root
 }
 
-function create_ingress_l7_router {
-
-    local lb_port_id
-    local lb_name
-    local project_id
-    local max_timeout
-    local lb_vip
-    local fake_svc_name
-    local l7_router_fip
-    local project_id
-    local lb_uuid
-
-    lb_name=${KURYR_L7_ROUTER_NAME}
-    max_timeout=1000
-    project_id=$(get_or_create_project \
-        "$KURYR_NEUTRON_DEFAULT_PROJECT" default)
-
-    create_load_balancer "$lb_name" "$KURYR_NEUTRON_DEFAULT_SERVICE_SUBNET" "$project_id"
-
-    wait_for_lb $lb_name $max_timeout
-
-    lb_port_id="$(get_loadbalancer_attribute "$lb_name" "vip_port_id")"
-
-    #allocate FIP and bind it to lb vip
-    l7_router_fip=$(openstack --os-cloud devstack-admin \
-           --os-region "$REGION_NAME" \
-           floating ip create --project "$project_id" \
-            --subnet "$KURYR_NEUTRON_DEFAULT_EXT_SVC_SUBNET" \
-             "$KURYR_NEUTRON_DEFAULT_EXT_SVC_NET" \
-            -f value -c floating_ip_address)
-
-    openstack  --os-cloud devstack-admin \
-            --os-region "$REGION_NAME" \
-            floating ip set --port "$lb_port_id" "$l7_router_fip"
-
-    lb_uuid="$(get_loadbalancer_attribute "$lb_name" "id")"
-    iniset "$KURYR_CONFIG" ingress l7_router_uuid "$lb_uuid"
-
-    #in case tempest enabled, update router's FIP in tempest.conf
-    if is_service_enabled tempest; then
-       iniset $TEMPEST_CONFIG kuryr_kubernetes ocp_router_fip "$l7_router_fip"
-    fi
-
-    if is_service_enabled octavia; then
-        echo -n "Octavia: no need to create fake k8s service for Ingress."
-    else
-        # keep fake an endpoint less k8s service to keep Kubernetes API server
-        # from allocating ingress LB vip
-        fake_svc_name='kuryr-svc-ingress'
-        echo -n "LBaaS: create fake k8s service: $fake_svc_name for Ingress."
-        lb_vip="$(get_loadbalancer_attribute "$lb_name" "vip_address")"
-        create_k8s_fake_service $fake_svc_name $lb_vip
-    fi
-}
 
 function configure_overcloud_vm_k8s_svc_sg {
     local dst_port
@@ -992,7 +917,7 @@ function configure_overcloud_vm_k8s_svc_sg {
         awk '{if ($2=="default") print $1}')
     openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
         security group rule create --project "$project_id" \
-        --dst-port "$dst_port" "$security_group"
+        --dst-port "$dst_port" --ethertype "$KURYR_ETHERTYPE" "$security_group"
     openstack port set "$KURYR_OVERCLOUD_VM_PORT" --security-group service_pod_access
 }
 
@@ -1004,8 +929,8 @@ function update_tempest_conf_file {
     if [[ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]]; then
         iniset $TEMPEST_CONFIG kuryr_kubernetes containerized True
     fi
-    if [[ "$KURYR_SG_DRIVER" == "namespace" ]] && [[ "$KURYR_SUBNET_DRIVER" == "namespace" ]]; then
-        iniset $TEMPEST_CONFIG kuryr_kubernetes namespace_enabled True
+    if [[ "$KURYR_SUBNET_DRIVER" == "namespace" ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes subnet_per_namespace True
     fi
     if [[ "$KURYR_K8S_SERIAL_TESTS" == "True" ]]; then
         iniset $TEMPEST_CONFIG kuryr_kubernetes run_tests_serial True
@@ -1028,7 +953,14 @@ function update_tempest_conf_file {
     if [[ "$KURYR_K8S_CLOUD_PROVIDER" == "True" ]]; then
         iniset $TEMPEST_CONFIG kuryr_kubernetes cloud_provider True
     fi
+    if [[ "$KURYR_CONFIGMAP_MODIFIABLE" == "True" ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes configmap_modifiable True
+    fi
+    if [[ "$KURYR_IPV6" == "True" ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes ipv6 True
+    fi
     iniset $TEMPEST_CONFIG kuryr_kubernetes validate_crd True
+    iniset $TEMPEST_CONFIG kuryr_kubernetes kuryrnetworks True
 }
 
 source $DEST/kuryr-kubernetes/devstack/lib/kuryr_kubernetes
@@ -1069,6 +1001,8 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
                 KURYR_K8S_API_ROOT="https://${k8s_api_clusterip}:${KURYR_K8S_API_LB_PORT}"
             fi
             iniset "$KURYR_CONFIG" kubernetes api_root ${KURYR_K8S_API_ROOT}
+            iniset "$KURYR_CONFIG" kubernetes ssl_ca_crt_file '""'
+            iniset "$KURYR_CONFIG" kubernetes token_file '""'
         else
             iniset "$KURYR_CONFIG" kubernetes api_root '""'
         fi
@@ -1165,8 +1099,7 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
     fi
 
     if is_service_enabled kuryr-kubernetes; then
-        /usr/local/bin/kubectl apply -f ${KURYR_HOME}/kubernetes_crds/kuryrnet.yaml
-        /usr/local/bin/kubectl apply -f ${KURYR_HOME}/kubernetes_crds/kuryrnetpolicy.yaml
+        /usr/local/bin/kubectl apply -f ${KURYR_HOME}/kubernetes_crds/kuryr_crds/
         if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
             generate_containerized_kuryr_resources
         fi
@@ -1179,12 +1112,6 @@ elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
     if is_service_enabled kuryr-kubernetes; then
         if is_service_enabled octavia; then
             create_k8s_api_service
-            #create Ingress L7 router if required
-            enable_ingress=$(trueorfalse False KURYR_ENABLE_INGRESS)
-
-            if [ "$enable_ingress" == "True" ]; then
-                create_ingress_l7_router
-            fi
         fi
 
         # FIXME(dulek): This is a very late phase to start Kuryr services.
@@ -1261,3 +1188,6 @@ if [[ "$1" == "unstack" ]]; then
 
     cleanup_kuryr_devstack_iptables
 fi
+
+# Restore xtrace
+$XTRACE
