@@ -12,17 +12,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
+import munch
+from openstack import exceptions as os_exc
+from unittest import mock
 
-from kuryr_kubernetes import constants
 from kuryr_kubernetes.controller.drivers import network_policy
 from kuryr_kubernetes import exceptions
 from kuryr_kubernetes.tests import base as test_base
 from kuryr_kubernetes.tests.unit import kuryr_fixtures as k_fix
 from kuryr_kubernetes import utils
 
-from neutronclient.common import exceptions as n_exc
-
 
 def get_pod_obj():
     return {
@@ -52,6 +51,21 @@ def get_pod_obj():
         }}
 
 
+def get_namespace_obj():
+    return {
+        "kind": "Namespace",
+        "metadata": {
+            "annotations": {
+                "openstack.org/kuryr-namespace-label":
+                    "{\"projetc\": \"myproject\"}",
+                "openstack.org/kuryr-net-crd": "ns-myproject"
+            },
+            "labels": {
+                "project": "myproject"
+            },
+            "name": "myproject"}}
+
+
 class TestNetworkPolicyDriver(test_base.TestCase):
 
     def setUp(self):
@@ -65,14 +79,14 @@ class TestNetworkPolicyDriver(test_base.TestCase):
         self._e_rules = [{'security_group_rule': {'id': ''}}]
 
         self._policy = {
-            'apiVersion': u'networking.k8s.io/v1',
-            'kind': u'NetworkPolicy',
+            'apiVersion': 'networking.k8s.io/v1',
+            'kind': 'NetworkPolicy',
             'metadata': {
                 'name': self._policy_name,
-                'resourceVersion': u'2259309',
+                'resourceVersion': '2259309',
                 'generation': 1,
-                'creationTimestamp': u'2018-09-18T14:09:51Z',
-                'namespace': u'default',
+                'creationTimestamp': '2018-09-18T14:09:51Z',
+                'namespace': 'default',
                 'annotations': {},
                 'selfLink': self._policy_link,
                 'uid': self._policy_uid
@@ -96,7 +110,7 @@ class TestNetworkPolicyDriver(test_base.TestCase):
 
         self._crd = {
             'metadata': {'name': mock.sentinel.name,
-                         'namespace': u'default',
+                         'namespace': 'default',
                          'selfLink': mock.sentinel.selfLink},
             'spec': {
                 'egressSgRules': [
@@ -125,7 +139,7 @@ class TestNetworkPolicyDriver(test_base.TestCase):
                 'securityGroupId': self._sg_id,
                 'securityGroupName': mock.sentinel.sg_name}}
 
-        self.neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.neutron = self.useFixture(k_fix.MockNetworkClient()).client
         self.kubernetes = self.useFixture(k_fix.MockK8sClient()).client
         self._driver = network_policy.NetworkPolicyDriver()
 
@@ -223,14 +237,13 @@ class TestNetworkPolicyDriver(test_base.TestCase):
                                                              m_add_crd,
                                                              m_get_crd,
                                                              m_add_default):
-        self._driver.neutron.create_security_group.return_value = {
-            'security_group': {'id': mock.sentinel.id,
-                               'security_group_rules': []}}
-        m_utils.get_subnet_cidr.return_value = {
-            'subnet': {'cidr': mock.sentinel.cidr}}
+        self._driver.os_net.create_security_group.return_value = (
+            munch.Munch({'id': mock.sentinel.id,
+                         'security_group_rules': []}))
+        m_utils.get_subnet_cidr.return_value = {'cidr': mock.sentinel.cidr}
         m_parse.return_value = (self._i_rules, self._e_rules)
-        self._driver.neutron.create_security_group_rule.return_value = {
-            'security_group_rule': {'id': mock.sentinel.id}}
+        self._driver.os_net.create_security_group_rule.return_value = (
+            munch.Munch({'id': mock.sentinel.id}))
         self._driver.create_security_group_rules_from_network_policy(
             self._policy, self._project_id)
         m_get_crd.assert_called_once()
@@ -249,15 +262,14 @@ class TestNetworkPolicyDriver(test_base.TestCase):
     def test_create_security_group_rules_with_k8s_exc(self, m_utils, m_parse,
                                                       m_add_crd, m_get_crd,
                                                       m_add_default):
-        self._driver.neutron.create_security_group.return_value = {
-            'security_group': {'id': mock.sentinel.id,
-                               'security_group_rules': []}}
-        m_utils.get_subnet_cidr.return_value = {
-            'subnet': {'cidr': mock.sentinel.cidr}}
+        self._driver.os_net.create_security_group.return_value = (
+            munch.Munch({'id': mock.sentinel.id,
+                         'security_group_rules': []}))
+        m_utils.get_subnet_cidr.return_value = {'cidr': mock.sentinel.cidr}
         m_parse.return_value = (self._i_rules, self._e_rules)
         m_get_crd.side_effect = exceptions.K8sClientException
-        self._driver.neutron.create_security_group_rule.return_value = {
-            'security_group_rule': {'id': mock.sentinel.id}}
+        self._driver.os_net.create_security_group_rule.return_value = (
+            munch.Munch({'id': mock.sentinel.id}))
         self.assertRaises(
             exceptions.K8sClientException,
             self._driver.create_security_group_rules_from_network_policy,
@@ -277,15 +289,14 @@ class TestNetworkPolicyDriver(test_base.TestCase):
     def test_create_security_group_rules_error_add_crd(self, m_utils, m_parse,
                                                        m_add_crd, m_get_crd,
                                                        m_add_default):
-        self._driver.neutron.create_security_group.return_value = {
-            'security_group': {'id': mock.sentinel.id,
-                               'security_group_rules': []}}
-        m_utils.get_subnet_cidr.return_value = {
-            'subnet': {'cidr': mock.sentinel.cidr}}
+        self._driver.os_net.create_security_group.return_value = (
+            munch.Munch({'id': mock.sentinel.id,
+                         'security_group_rules': []}))
+        m_utils.get_subnet_cidr.return_value = {'cidr': mock.sentinel.cidr}
         m_parse.return_value = (self._i_rules, self._e_rules)
         m_add_crd.side_effect = exceptions.K8sClientException
-        self._driver.neutron.create_security_group_rule.return_value = {
-            'security_group_rule': {'id': mock.sentinel.id}}
+        self._driver.os_net.create_security_group_rule.return_value = (
+            munch.Munch({'id': mock.sentinel.id}))
         self.assertRaises(
             exceptions.K8sClientException,
             self._driver.create_security_group_rules_from_network_policy,
@@ -294,10 +305,10 @@ class TestNetworkPolicyDriver(test_base.TestCase):
         m_add_default.assert_called_once()
 
     def test_create_security_group_rules_with_n_exc(self):
-        self._driver.neutron.create_security_group.side_effect = (
-            n_exc.NeutronClientException())
+        self._driver.os_net.create_security_group.side_effect = (
+            os_exc.SDKException())
         self.assertRaises(
-            n_exc.NeutronClientException,
+            os_exc.SDKException,
             self._driver.create_security_group_rules_from_network_policy,
             self._policy, self._project_id)
 
@@ -335,88 +346,104 @@ class TestNetworkPolicyDriver(test_base.TestCase):
             self._policy)
         m_parse.assert_called_with(self._policy, self._sg_id)
 
-    def test_get_namespaces_cidr(self):
-        namespace_selector = {'matchLabels': {'test': 'test'}}
-        pod = get_pod_obj()
-        annotation = mock.sentinel.annotation
-        subnet_cidr = mock.sentinel.subnet_cidr
-        net_crd = {'spec': {'subnetCIDR': subnet_cidr}}
-        pod['metadata']['annotations'][constants.K8S_ANNOTATION_NET_CRD] = (
-            annotation)
-        self.kubernetes.get.side_effect = [{'items': [pod]}, net_crd]
+    def test_get_namespaces(self):
+        namespace_selector = {'namespaceSelector': {
+                              'matchLabels': {'project': 'myproject'}}}
+        self.kubernetes.get.side_effect = [{'items': [get_namespace_obj()]}]
 
-        resp = self._driver._get_namespaces_cidr(namespace_selector)
-        self.assertEqual(subnet_cidr, resp[0].get('cidr'))
+        resp = self._driver._get_namespaces(namespace_selector)
+        self.assertEqual([get_namespace_obj()], resp)
         self.kubernetes.get.assert_called()
 
-    def test_get_namespaces_cidr_no_matches(self):
+    def test_get_namespaces_no_matches(self):
         namespace_selector = {'matchLabels': {'test': 'test'}}
         self.kubernetes.get.return_value = {'items': []}
 
-        resp = self._driver._get_namespaces_cidr(namespace_selector)
+        resp = self._driver._get_namespaces(namespace_selector)
         self.assertEqual([], resp)
         self.kubernetes.get.assert_called_once()
 
-    def test_get_namespaces_cidr_no_annotations(self):
-        namespace_selector = {'matchLabels': {'test': 'test'}}
-        pod = get_pod_obj()
-        self.kubernetes.get.return_value = {'items': [pod]}
-
-        self.assertRaises(exceptions.ResourceNotReady,
-                          self._driver._get_namespaces_cidr,
-                          namespace_selector)
-        self.kubernetes.get.assert_called_once()
-
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_services')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
-                       '_get_namespaces_cidr')
+                       '_get_resource_details')
+    @mock.patch.object(network_policy.NetworkPolicyDriver,
+                       '_get_namespaces')
     @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
                 'create_security_group_rule_body')
-    def test_parse_network_policy_rules_with_rules(self, m_create,
-                                                   m_get_ns_cidr):
+    def test_parse_network_policy_rules_with_rules(
+            self, m_create, m_get_namespaces,
+            m_get_resource_details, m_get_svcs):
         subnet_cidr = '10.10.0.0/24'
-        m_get_ns_cidr.return_value = [{'cidr': subnet_cidr, 'namespace': ''}]
+        namespace = 'myproject'
+        m_get_namespaces.return_value = [get_namespace_obj()]
+        m_get_resource_details.return_value = subnet_cidr, namespace
         self._driver.parse_network_policy_rules(self._policy, self._sg_id)
+        m_get_namespaces.assert_called()
+        m_get_resource_details.assert_called()
         m_create.assert_called()
-        m_get_ns_cidr.assert_called()
 
     @mock.patch.object(network_policy.NetworkPolicyDriver,
-                       '_get_namespaces_cidr')
+                       '_get_namespaces')
     @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
                 'create_security_group_rule_body')
     def test_parse_network_policy_rules_with_no_rules(self, m_create,
-                                                      m_get_ns_cidr):
+                                                      m_get_ns):
         policy = self._policy.copy()
         policy['spec']['ingress'] = [{}]
         policy['spec']['egress'] = [{}]
         self._driver.parse_network_policy_rules(policy, self._sg_id)
-        m_get_ns_cidr.assert_not_called()
+        m_get_ns.assert_not_called()
         calls = [mock.call(self._sg_id, 'ingress'),
                  mock.call(self._sg_id, 'egress')]
         m_create.assert_has_calls(calls)
 
     @mock.patch.object(network_policy.NetworkPolicyDriver,
-                       '_get_namespaces_cidr')
-    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
-                'create_security_group_rule_body')
-    def test_parse_network_policy_rules_with_no_pod_selector(self, m_create,
-                                                             m_get_ns_cidr):
+                       '_create_all_pods_sg_rules')
+    def test_parse_network_policy_rules_with_no_pod_selector(
+            self, m_create_all_pods_sg_rules):
         policy = self._policy.copy()
         policy['spec']['ingress'] = [{'ports':
                                       [{'port': 6379, 'protocol': 'TCP'}]}]
         policy['spec']['egress'] = [{'ports':
                                      [{'port': 6379, 'protocol': 'TCP'}]}]
         self._driver.parse_network_policy_rules(policy, self._sg_id)
-        m_create.assert_called()
-        m_get_ns_cidr.assert_not_called()
+        m_create_all_pods_sg_rules.assert_called()
 
     @mock.patch.object(network_policy.NetworkPolicyDriver,
-                       '_get_namespaces_cidr')
+                       '_create_sg_rule_on_number_port')
+    @mock.patch.object(network_policy.NetworkPolicyDriver,
+                       '_get_namespaces')
+    def test_parse_network_policy_rules_with_ipblock(self,
+                                                     m_get_namespaces,
+                                                     m_create_sg_rule):
+        policy = self._policy.copy()
+        policy['spec']['ingress'] = [{'from':
+                                      [{'ipBlock':
+                                        {'cidr': '172.17.0.0/16',
+                                         'except': ['172.17.1.0/24']}}],
+                                      'ports': [{'port': 6379,
+                                                 'protocol': 'TCP'}]}]
+        policy['spec']['egress'] = [{'ports': [{'port': 5978, 'protocol':
+                                                'TCP'}],
+                                     'to': [{'ipBlock':
+                                             {'cidr': '10.0.0.0/24'}}]}]
+        self._driver.parse_network_policy_rules(policy, self._sg_id)
+        m_create_sg_rule.assert_called()
+
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_services')
+    @mock.patch.object(network_policy.NetworkPolicyDriver,
+                       '_get_resource_details')
+    @mock.patch.object(network_policy.NetworkPolicyDriver,
+                       '_get_namespaces')
     @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
                 'create_security_group_rule_body')
-    def test_parse_network_policy_rules_with_no_ports(self, m_create,
-                                                      m_get_ns_cidr):
+    def test_parse_network_policy_rules_with_no_ports(
+            self, m_create, m_get_namespaces, m_get_resource_details,
+            m_get_svcs):
         subnet_cidr = '10.10.0.0/24'
-        m_get_ns_cidr.return_value = [{'cidr': subnet_cidr, 'namespace': ''}]
+        namespace = 'myproject'
+        m_get_namespaces.return_value = [get_namespace_obj()]
+        m_get_resource_details.return_value = subnet_cidr, namespace
         policy = self._policy.copy()
         selectors = {'namespaceSelector': {
                      'matchLabels': {
@@ -429,13 +456,14 @@ class TestNetworkPolicyDriver(test_base.TestCase):
                 [selectors]}]
         selectors = {'namespace_selector': selectors['namespaceSelector']}
         self._driver.parse_network_policy_rules(policy, self._sg_id)
-        m_get_ns_cidr.assert_called()
+        m_get_namespaces.assert_called()
+        m_get_resource_details.assert_called()
         calls = [mock.call(self._sg_id, 'ingress', port_range_min=1,
                            port_range_max=65535, cidr=subnet_cidr,
-                           namespace=''),
+                           namespace=namespace),
                  mock.call(self._sg_id, 'egress', port_range_min=1,
                            port_range_max=65535, cidr=subnet_cidr,
-                           namespace='')]
+                           namespace=namespace)]
         m_create.assert_has_calls(calls)
 
     def test_knps_on_namespace(self):
