@@ -14,7 +14,7 @@
 #    under the License.
 
 import itertools
-import mock
+from unittest import mock
 import uuid
 
 import os_vif.objects.network as osv_network
@@ -356,42 +356,15 @@ class FakeLBaaSDriver(drv_base.LBaaSDriver):
     def get_loadbalancer_pool_name(self, lb_name, namespace, svc_name):
         return "%s/%s/%s" % (lb_name, namespace, svc_name)
 
-    def ensure_l7_policy(self, namespace, route_name,
-                         loadbalancer, pool,
-                         listener_id):
-        pass
-
-    def release_l7_policy(self, loadbalancer, l7_policy):
-        pass
-
-    def ensure_l7_rule(self, loadbalancer, l7_policy, compare_type,
-                       type, value):
-        pass
-
-    def release_l7_rule(self, loadbalancer, l7_rule):
-        pass
-
-    def update_l7_rule(self, l7_rule, new_value):
-        pass
-
-    def ensure_pool_attached_to_lb(self, loadbalancer, namespace,
-                                   svc_name, protocol):
-        return obj_lbaas.LBaaSPool(id=str(uuid.uuid4()),
-                                   loadbalancer_id=loadbalancer.id,
-                                   project_id=loadbalancer.project_id,
-                                   protocol=protocol)
-
-    def get_pool_by_name(self, pool_name, project_id):
-        pass
-
-    def get_lb_by_uuid(self, vip):
-        pass
-
-    def is_pool_used_by_other_l7policies(l7policy, pool):
-        pass
-
 
 class TestLoadBalancerHandler(test_base.TestCase):
+
+    @mock.patch('kuryr_kubernetes.controller.drivers.base.'
+                'ServiceProjectDriver.get_instance')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base.'
+                'ServiceSecurityGroupsDriver.get_instance')
+    @mock.patch('kuryr_kubernetes.controller.handlers.lbaas'
+                '.LoadBalancerHandler._cleanup_leftover_lbaas')
     @mock.patch('kuryr_kubernetes.config.CONF')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.ServicePubIpDriver.get_instance')
@@ -402,11 +375,15 @@ class TestLoadBalancerHandler(test_base.TestCase):
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
     def test_init(self, m_get_drv_lbaas, m_get_drv_project,
-                  m_get_drv_subnets, m_get_drv_service_pub_ip, m_cfg):
+                  m_get_drv_subnets, m_get_drv_service_pub_ip, m_cfg,
+                  m_cleanup_leftover_lbaas,
+                  m_get_svc_sg_drv, m_get_svc_drv_project):
         m_get_drv_lbaas.return_value = mock.sentinel.drv_lbaas
         m_get_drv_project.return_value = mock.sentinel.drv_project
         m_get_drv_subnets.return_value = mock.sentinel.drv_subnets
         m_get_drv_service_pub_ip.return_value = mock.sentinel.drv_lb_ip
+        m_get_svc_drv_project.return_value = mock.sentinel.drv_svc_project
+        m_get_svc_sg_drv.return_value = mock.sentinel.drv_sg
         m_cfg.kubernetes.endpoints_driver_octavia_provider = 'default'
         handler = h_lbaas.LoadBalancerHandler()
 
@@ -414,8 +391,16 @@ class TestLoadBalancerHandler(test_base.TestCase):
         self.assertEqual(mock.sentinel.drv_project, handler._drv_pod_project)
         self.assertEqual(mock.sentinel.drv_subnets, handler._drv_pod_subnets)
         self.assertEqual(mock.sentinel.drv_lb_ip, handler._drv_service_pub_ip)
+        self.assertEqual(mock.sentinel.drv_svc_project, handler._drv_project)
+        self.assertEqual(mock.sentinel.drv_sg, handler._drv_sg)
         self.assertIsNone(handler._lb_provider)
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.base.'
+                'ServiceProjectDriver.get_instance')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base.'
+                'ServiceSecurityGroupsDriver.get_instance')
+    @mock.patch('kuryr_kubernetes.controller.handlers.lbaas'
+                '.LoadBalancerHandler._cleanup_leftover_lbaas')
     @mock.patch('kuryr_kubernetes.config.CONF')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.ServicePubIpDriver.get_instance')
@@ -427,11 +412,14 @@ class TestLoadBalancerHandler(test_base.TestCase):
                 '.LBaaSDriver.get_instance')
     def test_init_provider_ovn(self, m_get_drv_lbaas, m_get_drv_project,
                                m_get_drv_subnets, m_get_drv_service_pub_ip,
-                               m_cfg):
+                               m_cfg, m_cleanup_leftover_lbaas,
+                               m_get_svc_sg_drv, m_get_svc_drv_project):
         m_get_drv_lbaas.return_value = mock.sentinel.drv_lbaas
         m_get_drv_project.return_value = mock.sentinel.drv_project
         m_get_drv_subnets.return_value = mock.sentinel.drv_subnets
         m_get_drv_service_pub_ip.return_value = mock.sentinel.drv_lb_ip
+        m_get_svc_drv_project.return_value = mock.sentinel.drv_svc_project
+        m_get_svc_sg_drv.return_value = mock.sentinel.drv_sg
         m_cfg.kubernetes.endpoints_driver_octavia_provider = 'ovn'
         handler = h_lbaas.LoadBalancerHandler()
 
@@ -439,6 +427,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
         self.assertEqual(mock.sentinel.drv_project, handler._drv_pod_project)
         self.assertEqual(mock.sentinel.drv_subnets, handler._drv_pod_subnets)
         self.assertEqual(mock.sentinel.drv_lb_ip, handler._drv_service_pub_ip)
+        self.assertEqual(mock.sentinel.drv_svc_project, handler._drv_project)
+        self.assertEqual(mock.sentinel.drv_sg, handler._drv_sg)
         self.assertEqual('ovn', handler._lb_provider)
 
     @mock.patch('kuryr_kubernetes.utils.get_lbaas_spec')
@@ -742,6 +732,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
             for member in state.members)
         return observed_targets
 
+    @mock.patch('kuryr_kubernetes.controller.handlers.lbaas'
+                '.LoadBalancerHandler._cleanup_leftover_lbaas')
     @mock.patch('kuryr_kubernetes.controller.handlers.lbaas.'
                 'LoadBalancerHandler._sync_lbaas_sgs')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
@@ -751,7 +743,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
     def test_sync_lbaas_members(self, m_get_drv_lbaas, m_get_drv_project,
-                                m_get_drv_subnets, m_sync_lbaas_sgs):
+                                m_get_drv_subnets, m_sync_lbaas_sgs,
+                                m_cleanup_leftover_lbaas):
         # REVISIT(ivc): test methods separately and verify ensure/release
         project_id = str(uuid.uuid4())
         subnet_id = str(uuid.uuid4())
@@ -778,6 +771,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
         self.assertEqual(sorted(expected_targets.items()), observed_targets)
         self.assertEqual(expected_ip, str(state.loadbalancer.ip))
 
+    @mock.patch('kuryr_kubernetes.controller.handlers.lbaas'
+                '.LoadBalancerHandler._cleanup_leftover_lbaas')
     @mock.patch('kuryr_kubernetes.controller.handlers.lbaas.'
                 'LoadBalancerHandler._sync_lbaas_sgs')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
@@ -788,7 +783,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
                 '.LBaaSDriver.get_instance')
     def test_sync_lbaas_members_udp(self, m_get_drv_lbaas,
                                     m_get_drv_project, m_get_drv_subnets,
-                                    m_sync_lbaas_sgs):
+                                    m_sync_lbaas_sgs,
+                                    m_cleanup_leftover_lbaas):
         # REVISIT(ivc): test methods separately and verify ensure/release
         project_id = str(uuid.uuid4())
         subnet_id = str(uuid.uuid4())
@@ -815,6 +811,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
         self.assertEqual([], observed_targets)
         self.assertEqual(expected_ip, str(state.loadbalancer.ip))
 
+    @mock.patch('kuryr_kubernetes.controller.handlers.lbaas'
+                '.LoadBalancerHandler._cleanup_leftover_lbaas')
     @mock.patch('kuryr_kubernetes.controller.handlers.lbaas.'
                 'LoadBalancerHandler._sync_lbaas_sgs')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
@@ -825,7 +823,7 @@ class TestLoadBalancerHandler(test_base.TestCase):
                 '.LBaaSDriver.get_instance')
     def test_sync_lbaas_members_svc_listener_port_edit(
             self, m_get_drv_lbaas, m_get_drv_project, m_get_drv_subnets,
-            m_sync_lbaas_sgs):
+            m_sync_lbaas_sgs, m_cleanup_leftover_lbaas):
         # REVISIT(ivc): test methods separately and verify ensure/release
         project_id = str(uuid.uuid4())
         subnet_id = str(uuid.uuid4())
@@ -872,6 +870,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
         self.skipTest("skipping until generalised annotation handling is "
                       "implemented")
 
+    @mock.patch('kuryr_kubernetes.controller.handlers.lbaas'
+                '.LoadBalancerHandler._cleanup_leftover_lbaas')
     @mock.patch('kuryr_kubernetes.controller.handlers.lbaas.'
                 'LoadBalancerHandler._sync_lbaas_sgs')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
@@ -882,7 +882,7 @@ class TestLoadBalancerHandler(test_base.TestCase):
                 '.LBaaSDriver.get_instance')
     def test_add_new_members_udp(self, m_get_drv_lbaas,
                                  m_get_drv_project, m_get_drv_subnets,
-                                 m_sync_lbaas_sgs):
+                                 m_sync_lbaas_sgs, m_cleanup_leftover_lbaas):
         project_id = str(uuid.uuid4())
         subnet_id = str(uuid.uuid4())
         current_ip = '1.1.1.1'
