@@ -16,7 +16,7 @@ import time
 
 import requests
 
-from neutronclient.common import exceptions as n_exc
+from openstack import exceptions as os_exc
 from os_vif import objects
 from oslo_cache import core as cache
 from oslo_config import cfg
@@ -36,11 +36,12 @@ LOG = log.getLogger(__name__)
 VALID_MULTI_POD_POOLS_OPTS = {'noop': ['neutron-vif',
                                        'nested-vlan',
                                        'nested-macvlan',
-                                       'sriov'],
+                                       'sriov',
+                                       'nested-dpdk'],
                               'neutron': ['neutron-vif'],
                               'nested': ['nested-vlan'],
                               }
-DEFAULT_TIMEOUT = 180
+DEFAULT_TIMEOUT = 500
 DEFAULT_INTERVAL = 3
 
 subnet_caching_opts = [
@@ -162,45 +163,53 @@ def get_leader_name():
 def get_nodes_ips():
     """Get the IPs of the trunk ports associated to the deployment."""
     trunk_ips = []
-    neutron = clients.get_neutron_client()
+    os_net = clients.get_network_client()
     tags = CONF.neutron_defaults.resource_tags
     if tags:
-        ports = neutron.list_ports(status='ACTIVE',
-                                   tags=CONF.neutron_defaults.resource_tags)
+        ports = os_net.ports(status='ACTIVE', tags=tags)
     else:
         # NOTE(ltomasbo: if tags are not used, assume all the trunk ports are
         # part of the kuryr deployment
-        ports = neutron.list_ports(status='ACTIVE')
-    for port in ports.get('ports'):
-        if port.get('trunk_details'):
-            trunk_ips.append(port['fixed_ips'][0]['ip_address'])
+        ports = os_net.ports(status='ACTIVE')
+    for port in ports:
+        if port.trunk_details:
+            trunk_ips.append(port.fixed_ips[0]['ip_address'])
     return trunk_ips
 
 
 @MEMOIZE
 def get_subnet(subnet_id):
-    neutron = clients.get_neutron_client()
+    os_net = clients.get_network_client()
 
-    n_subnet = neutron.show_subnet(subnet_id).get('subnet')
-    network_id = n_subnet['network_id']
-    n_network = neutron.show_network(network_id).get('network')
+    n_subnet = os_net.get_subnet(subnet_id)
+    n_network = os_net.get_network(n_subnet.network_id)
 
     subnet = os_vif_util.neutron_to_osvif_subnet(n_subnet)
     network = os_vif_util.neutron_to_osvif_network(n_network)
     network.subnets.objects.append(subnet)
-
     return network
 
 
 @MEMOIZE
 def get_subnet_cidr(subnet_id):
-    neutron = clients.get_neutron_client()
+    os_net = clients.get_network_client()
     try:
-        subnet_obj = neutron.show_subnet(subnet_id)
-    except n_exc.NeutronClientException:
+        subnet_obj = os_net.get_subnet(subnet_id)
+    except os_exc.ResourceNotFound:
         LOG.exception("Subnet %s CIDR not found!", subnet_id)
         raise
-    return subnet_obj.get('subnet')['cidr']
+    return subnet_obj.cidr
+
+
+@MEMOIZE
+def get_subnetpool_version(subnetpool_id):
+    os_net = clients.get_network_client()
+    try:
+        subnetpool_obj = os_net.get_subnet_pool(subnetpool_id)
+    except os_exc.ResourceNotFound:
+        LOG.exception("Subnetpool %s not found!", subnetpool_id)
+        raise
+    return subnetpool_obj.ip_version
 
 
 def extract_pod_annotation(annotation):
@@ -219,12 +228,11 @@ def extract_pod_annotation(annotation):
 
 def has_limit(quota):
     NO_LIMIT = -1
-    return quota != NO_LIMIT
+    return quota['limit'] != NO_LIMIT
 
 
-def is_available(resource, resource_quota, neutron_func):
-    qnt_resources = len(neutron_func().get(resource))
-    availability = resource_quota - qnt_resources
+def is_available(resource, resource_quota):
+    availability = resource_quota['limit'] - resource_quota['used']
     if availability <= 0:
         LOG.error("Quota exceeded for resource: %s", resource)
         return False
@@ -323,9 +331,8 @@ def get_endpoints_link(service):
     link_parts = svc_link.split('/')
 
     if link_parts[-2] != 'services':
-        raise exceptions.IntegrityError(_(
-            "Unsupported service link: %(link)s") % {
-            'link': svc_link})
+        raise exceptions.IntegrityError(
+            f"Unsupported service link: {svc_link}")
     link_parts[-2] = 'endpoints'
 
     return "/".join(link_parts)
@@ -357,3 +364,15 @@ def get_service_ports(service):
              'port': port['port'],
              'targetPort': str(port['targetPort'])}
             for port in service['spec']['ports']]
+
+
+@MEMOIZE
+def get_service_subnet_version():
+    os_net = clients.get_network_client()
+    svc_subnet_id = CONF.neutron_defaults.service_subnet
+    try:
+        svc_subnet = os_net.get_subnet(svc_subnet_id)
+    except os_exc.ResourceNotFound:
+        LOG.exception("Service subnet %s not found", svc_subnet_id)
+        raise
+    return svc_subnet.ip_version
